from .encoder import encode
from .decoder import decode
del base, decoder, encoder

def fencode(filepath: str, outpath:str=None, forcemode:bool=True) -> "FileIOWrapper":
    """
    Encoding a File at the Specified Path.

    Parameters
    ----------
    * filepath: Path of the File to be Encoded;
    * outpath: Path of the Output File, which Loads the Encoded Cipher;
    * forcemode;

    Returns
    -------
    * fout: The Text-IO Wrapper of the Output File.

    See Also
    --------
    `.crypto.fdecode(...)`

    Examples
    --------
    >>> import icsk
    >>> fp = "./speech.txt"
    >>> fl = open(fp, "r")
    >>> print(fl.read())
    Before I talk about the things we human beings like or love, let us make it clear that in different time, places, and conditions, we have different opinions on the same person or the same thing.
    ...
    When I listened this song for the first time, I was programming to learn the algorithm DFS, in short of Depth First Search. It's an algorithm based on the algorithm "recursion" we learnt yesterday. I couldn't process all the details in the program when I deal with the problem related to it. I got tired just like "ryo" used to. I met this song exactly the time I was to give up. With its strength inner, I tried to calm down and rearrange what I learnt. Finally I overcame the problem.
    >>> fl.close()
    >>> fl = icsk.crypto.fencode(fp)
    >>> fl
    <_io.TextIOWrapper name='./speech.txt.mikucrypto' mode='w' encoding='UTF-8'>
    >>> fl.close()
    >>> !cat ./speech.txt.mikucrypto
    icsk3ck9is3k9isci3skc9s3cki93k9cis3ci9k...
    """

    outpath = outpath or (filepath + ".mikucrypto")
    fout = open(outpath, "w" if forcemode else "x")

    with open(filepath, "r") as flin:
        fout.write(encode(flin.read()))
        flin.close()

    return fout

def fdecode(filepath: str, outpath:str=None, forcemode:bool=True) -> "FileIOWrapper":
    """
    Usages are Similar to `.crypto.fencode(...)`.
    """
    outpath = outpath or (filepath + ".txt")
    fout = open(outpath, "w" if forcemode else "x")
    with open(filepath, "r") as flin:
        fout.write(decode(flin.read()))
        flin.close()
    return fout



from .base import np, Constants

def strcomp(x: str) -> tuple:
    _r = len(x) % 3
    if _r:
        x += " " * (3 - _r)
    return x, _r

def str2arr(x: str) -> np.ndarray:
    return np.array([ Constants.MAP_CHAR_NUM[each] for each in x ]).reshape((len(x) // 3, 3))

def arrencode(x: np.ndarray) -> np.ndarray:
    return ((x @ Constants.MAT_KEY) % 128).flatten()

def num2cry(x: int) -> str:
    flag = False
    if x >= 64:
        flag = True
        x -= 64
    fnum = x // 24 + 1
    x %= 24
    res = Constants.LIST_ALL[x]
    res = res[ : fnum ] + "*" + res[ fnum : ]
    if flag:
        res = res.replace("*", "3") + "9"
    else:
        res = "3" + res.replace("*", "9")
    return res

def arr2cry(x: np.ndarray, r: int) -> str:
    res = ""
    for each in x:
        res += num2cry(each)
    res = Constants.LIST_ALL[0] + res + ["4ever", "4love", "4star"][r]
    return res

def encode(x: str) -> str:
    x, r = strcomp(x)
    x = str2arr(x)
    x = arrencode(x)
    x = arr2cry(x, r)
    return x



from .base import np, Constants

def cry2num(x: str) -> int:
    flag, fnum = False, 0
    if x[0] == "3":
        x = x[ 1 : ].replace("9", "*")
    else:
        flag = True
        x = x[ : (-1) ].replace("3", "*")
    for k in range(1, 4):
        if x[k] == "*":
            fnum = k - 1
            x = x[ : k ] + x[ (k + 1) : ]
            break
    res = 0
    for k in range(24):
        if x == Constants.LIST_ALL[k]:
            res = k
            break
    res += fnum * 24
    if flag:
        res += 64
    return res

def cry2arr(x: str) -> tuple:
    _r = {
        "4ever": 0,
        "4love": 2,
        "4star": 1,
        }[x[ (-5) : ]]
    return np.array([ cry2num(x[ k * 6 + 4 : k * 6 + 10 ]) for k in range((len(x) - 9) // 6) ]), _r

def arr2str(x: np.ndarray, r: int) -> str:
    x = (x.reshape((len(x) // 3, 3)) @ Constants.MAT_KEY_INV) % 128
    x = "".join(Constants.MAP_NUM_CHAR[each] for each in x.flatten())
    if r:
        x = x[ : (-r) ]
    return x

def decode(x: str) -> str:
    x, r = cry2arr(x)
    x = arr2str(x, r)
    return x



#!/usr/bin/env Python
# -*- coding: utf-8 -*-
import setuptools
setuptools.setup(
    name = "ichisaki",
    version = "0.0.8",
    description = "A Simple Toolkit just for FUN",
    long_description = open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type = "text/markdown",
    url = "https://github.com/sandyzikun/ichisaki.git",
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        ],
    packages = setuptools.find_packages(), #["icsk"],
    install_requires = ["numpy>=1.14.3"], # Actually it needs Functions in `np.linalg` which is in NumPy>=1.9.3
    )
